<?php

namespace Drupal\onesignal;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityTypeManagerInterface;

use GuzzleHttp\Client as GuzzleClient;
use Http\Adapter\Guzzle6\Client as GuzzleAdapter;
use Http\Client\Common\HttpMethodsClient as HttpClient;
use Http\Message\MessageFactory\GuzzleMessageFactory;
use OneSignal\Config;
use OneSignal\OneSignal;

/**
 * Class OneSignalService.
 */
class OneSignalService {

  /**
   * Drupal\Core\Entity\EntityTypeManager definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The config factory object.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * @var \OneSignal\OneSignal
   */
  private $oneSignalApi;

  /**
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ConfigFactory $config_factory) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * Get OneSignal App Id.
   *
   * @return string|null
   */
  public function getAppId(): ?string {
    return $this->configFactory->get('onesignal.config')->get('onesignal_app_id');
  }

  /**
   * Get OneSignal REST Api Key.
   *
   * @return string|null
   */
  public function getRestAPIKey(): ?string {
    return $this->configFactory->get('onesignal.config')->get('onesignal_rest_api_key');
  }

  /**
   * Get OneSignal API connector.
   *
   * @return \OneSignal\OneSignal
   */
  public function getApi(): OneSignal {
    if (!$this->oneSignalApi) {

      $apiConfig = new Config();

      if (!empty($this->getAppId())) {
        $apiConfig->setApplicationId($this->getAppId());
      }

      if (!empty($this->getRestAPIKey())) {
        $apiConfig->setApplicationAuthKey($this->getRestAPIKey());
      }

      // Guzzle config
      // http://docs.guzzlephp.org/en/stable/quickstart.html
      $guzzle = new GuzzleClient([
        'timeout' => 15,
        'read_timeout' => 15,
        'connect_timeout' => 10,
      ]);

      $client = new HttpClient(new GuzzleAdapter($guzzle), new GuzzleMessageFactory());
      $this->oneSignalApi = new OneSignal($apiConfig, $client);
    }

    return $this->oneSignalApi;
  }

  /**
   * View the details of all of your current OneSignal applications.
   *
   * @note Requires your OneSignal User Auth Key.
   *
   * @see https://documentation.onesignal.com/reference#view-apps-apps
   * @return array
   */
  public function getApps(): array {
    return $this->getApi()->apps->getAll();
  }

  /**
   * View the details of a single OneSignal application.
   *
   * @param string $app_id
   *
   * @see https://documentation.onesignal.com/reference#view-an-app
   * @return array
   */
  public function getSingleApp($app_id): array {
    return $this->getApi()->apps->getOne($app_id);
  }

  /**
   * View the details of multiple devices in one of your OneSignal apps.
   *
   * @see https://documentation.onesignal.com/reference#view-devices
   * @return array
   */
  public function getDevices(): array {
    return $this->getApi()->devices->getAll();
  }

  /**
   * View the details of an existing device in your configured OneSignal application.
   *
   * @param string $device_id
   *
   * @see https://documentation.onesignal.com/reference#view-device
   * @return array
   */
  public function getDevice($device_id): array {
    return $this->getApi()->devices->getOne($device_id);
  }

  /**
   * View the details of multiple notifications.
   *
   * @see https://documentation.onesignal.com/reference#view-notifications
   * @return array
   */
  public function getNotifications(): array {
    return $this->getApi()->notifications->getAll();
  }

  /**
   * Get the details of a single notification.
   *
   * @param string $id
   *
   * @see https://documentation.onesignal.com/reference#view-notification
   * @return array
   */
  public function getNotification($id): array {
    return $this->getApi()->notifications->getOne($id);
  }

  /**
   * Create and send notifications or emails to a segment or individual users.
   *
   * You may target users in one of three ways using this method: by Segment, by Filter, or by Device (at least one targeting parameter must be specified)
   *
   * @param array $data
   *
   * @return array
   * @see https://documentation.onesignal.com/reference#create-notification
   */
  public function addNotification(array $data = []): array {
    return $this->getApi()->notifications->add($data);
  }

  /**
   * Stop a scheduled or currently outgoing notification.
   *
   * @param $id
   *
   * @see https://documentation.onesignal.com/reference#cancel-notification
   * @return array
   */
  public function stopNotification($id): array {
    return $this->getApi()->notifications->cancel($id);
  }
}
